
/*
The circuit: 
 * Digital pin 2 of Crowduino works as RX connected to TX of USBSerial Adapter
 * Digital pin 3 of Crowduino works as TX connected to RX of USBSerial Adapter
 */
#include "U8glib.h"
#include "DHT.h"
//************************************************
#include <SoftwareSerial.h> 
#define DEBUG true
#define SSID "elecrow"
#define PASS "elecrow2014" 
SoftwareSerial sofSerial(2,3); // RX,TX
//****************************************************
const int buzzerPin = 4;    //buzzer connect to D4
const int LightPin = A0;    //light connect to A0
const int MoisturePin = A1;
const int DHTPIN =5;        //Temperature && Humidity sensor connect to D5    
#define DHTTYPE DHT11   // DHT 11 
#define Moisture 50  //Set the Moisture sensor alarm value
int LightValue=0;
int MoistureValue=0;
int HumidityValue=0;
int TemperatureValue=0;
String ip = "";
DHT dht(DHTPIN, DHTTYPE);
U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE);

void setup(void) {
  pinMode(buzzerPin, OUTPUT);
  dht.begin();  
  //************************************************
 //** ESP8266 init
  //*************************************************
  sofSerial.begin(9600);
  Serial.begin(115200); // your esp's baud rate might be different
 
  sendData("AT+CWMODE=3\r\n",1000,DEBUG); // configure as access point 
  sendData("AT+RST\r\n",2000,DEBUG); // reset module
 
       //connect to the wifi
       boolean connected=false;
       for(int i=0;i<5;i++)
       {
           if(connectWiFi())
           {
               connected = true;
                   break;
             }
         }
   //      if (!connected){while(1);}
         delay(2000);
         
  get_ip("AT+CIFSR\r\n",1000,DEBUG); // get ip address
  sendData("AT+CIPMUX=1\r\n",1000,DEBUG); // configure for multiple connections
  sendData("AT+CIPSERVER=1,2000\r\n",1000,DEBUG); // turn on server on port 2000
//  sendData("AT+CIPSTO=30\r\n",1000,DEBUG);
  //*************************************************
}

void loop(void)
{
   HumidityValue = dht.readHumidity();        //get Humidity
   TemperatureValue = dht.readTemperature();  // get Temperature
   MoistureValue = analogRead(MoisturePin);   //get Moisture
   LightValue= analogRead(LightPin);          //get  Light
    if(MoistureValue<Moisture) //if the MoistureValue lower than you set,if will alarm
    {
         buzzer(1000);  
    }
   u8g.firstPage();  //OLED Display
   do {
       OLED_display();
       u8g.setColorIndex(1);
   } while( u8g.nextPage() );
   
  //****************************************
  //*** TCP Clirnt connect and send data
  //**************************************
 if(Serial.available()) // check if the esp is sending a message 
  {
    if(Serial.find("+IPD,"))
    {
     delay(1000);
     int connectionId = Serial.read()-48; // subtract 48 because the read() function returns 
                                           // the ASCII decimal value and 0 (the first decimal number) starts at 48
       String webpage = "<h1>Hi all, I am the Elecrow Environment Tree,welcome to visit.";
       webpage += "<h1></br>Temperature:"+ String(TemperatureValue)+"C";
       webpage +="<h1></br>Humidity:"+String(HumidityValue)+"%";
       webpage +="<h1></br>Moisture:"+String(MoistureValue);
       webpage +="<h1></br>Light:"+String(LightValue);
       
       if(MoistureValue<Moisture)  
       {
            webpage += "<h1></br>help! help!!";
        }
        else
        {
            webpage += "<h1></br>Everything is fine, have a good job!";
        }
       webpage+= "</h1>";
     String cipSend = "AT+CIPSEND=";
     cipSend += connectionId;
     cipSend += ",";
     cipSend +=webpage.length();
     cipSend +="\r\n";   
     sendData(cipSend,1000,DEBUG);   
     sendData(webpage,1000,DEBUG);   //set data to the wep
 
      String closeCommand = "AT+CIPCLOSE="; //close the tcp client
     closeCommand+=5;
     closeCommand+="\r\n";    
     sendData(closeCommand,3000,DEBUG);
    }
  }
}


//*****************************************

String sendData(String command, const int timeout, boolean debug)
{
    String response = "";
    
    Serial.print(command); // send the read character to the Serial
    
    long int time = millis();
    
    while( (time+timeout) > millis())
    {
      while(Serial.available())
      {
        char c = Serial.read(); // read the next character.
        response+=c;
      }  
    }   
    if(debug)
    {
      sofSerial.print(response);
    } 
    return response;
}

boolean connectWiFi()
{
       String cmd="AT+CWJAP=\"";
       cmd+=SSID;
       cmd+="\",\"";
       cmd+=PASS;
       cmd+="\"";
       Serial.println(cmd);
       Serial.println(cmd);
       delay(3000);
       if(Serial.find("OK"))
     {
          sofSerial.println("OK, Connected to WiFi.");
     return true;
     }else
     {
           sofSerial.println("Can not connect to the WiFi.");
           return false;
     }
}

String get_ip(String command, const int timeout, boolean debug)
{
    int i=0;
    Serial.print(command); // send the read character to the Serial  
    long int time = millis(); 
    while( (time+timeout) > millis())
    {
      while(Serial.available())
      {
        char c = Serial.read(); // read the next character.       
       if(c=='"')
       {
           i++;
       }
       if((i>4)&&(i<6))
       {
          ip+=c;
       }   
   }         
 }
    ip+='"';    
    if(debug)
    {
      sofSerial.print("ip:");
      sofSerial.println(ip);
    }
    return ip;
}
//*************************************
void buzzer(int time)
{
    digitalWrite(buzzerPin, HIGH);
    delay(time);
    digitalWrite(buzzerPin, LOW);
}
void OLED_display(void) 
{
         if (isnan(TemperatureValue) || isnan(HumidityValue)) {
            u8g.setFont(u8g_font_7x14);
            u8g.drawStr(10,10,"Failed to read");
            u8g.drawStr(10,25,"from DHT");
        }else {
              u8g.setFont(u8g_font_7x14);
              u8g.setPrintPos(10, 10);
              u8g.print("T:");
              u8g.print(TemperatureValue, 1);
              u8g.print(" C");
              delay(1);
             u8g.setPrintPos(70, 10);
              u8g.print("H:");
              u8g.print(HumidityValue, 1);
              u8g.print(" %"); 
              delay(1);
        }
         u8g.setPrintPos(10, 25);             
         u8g.print("M:");
         u8g.print(MoistureValue, 1);
         delay(1);
         u8g.setPrintPos(70, 25);            
         u8g.print("L:");
         u8g.print(LightValue, 1);
         delay(1);      
         u8g.setPrintPos(10, 40);             
         u8g.print("IP:");
         u8g.setPrintPos(5, 55); 
         u8g.print(ip);
         delay(1);
}

